@@ -126,7 +126,7 @@ Itcl_InitRewriteEnsemble(
     Tcl_Interp *interp,
     int numRemoved,
     int numInserted,
-    int objc,
+    TCL_UNUSED(int) /* objc */,
     Tcl_Obj *const *objv)
 {
     return TclInitRewriteEnsemble(interp, numRemoved, numInserted, objv);
